@@ -7,15 +7,15 @@
 - make HTTP requests (jinko.makeRequest)
 """
 
+import sys
+
 from .__version__ import __version__
 import base64 as _base64
 import requests as _requests
 import getpass as _getpass
 import json as _json
 import os as _os
-import pandas as _pandas
-import sqlite3 as _sqlite3
-from typing import TypedDict as _TypedDict
+from typing import List, Any, Optional, TypedDict
 from urllib.parse import urlparse
 import warnings
 import tempfile
@@ -29,7 +29,7 @@ _baseUrl: str = "https://api.jinko.ai"
 _jinkoUrl: str = "https://jinko.ai"
 
 
-class CoreItemId(_TypedDict):
+class CoreItemIdDict(TypedDict):
     """Represents the CoreItem identifier.
 
     Attributes:
@@ -41,7 +41,7 @@ class CoreItemId(_TypedDict):
     snapshotId: str
 
 
-class MakeRequestOptions(_TypedDict):
+class MakeRequestOptions(TypedDict, total=False):
     """Additional options to use when making a request to the Jinko API.
 
     Attributes:
@@ -53,28 +53,28 @@ class MakeRequestOptions(_TypedDict):
         output_format (str): Expected content type of the response payload (may be ignored by server if not supported).
     """
 
-    name: str
-    description: str
-    folder_id: str
-    version_name: str
-    input_format: str = "application/json"
-    output_format: str
+    name: Optional[str]
+    description: Optional[str]
+    folder_id: Optional[str]
+    version_name: Optional[str]
+    input_format: Optional[str]
+    output_format: Optional[str]
 
 
-class ProjectItemInfoFromResponse(_TypedDict):
+class ProjectItemInfoFromResponse(TypedDict):
     """Informations contained in the "X-jinko-project-item" header returned
     by the Jinko API when creating/updating a ProjectItem (Modeling & Simulation).
 
     Attributes:
         sid (str): Short Id of the ProjectItem.
         description (str): Type of the ProjectItem.
-        coreItemId (CoreItemId): CoreItemId of the ProjectItem.
+        coreItemId (CoreItemIdDict): CoreItemI dictionnary of the ProjectItem.
         revision (int): Revision number of the ProjectItem.
     """
 
     sid: str
     kind: str
-    coreItemId: CoreItemId
+    coreItemId: CoreItemIdDict
     revision: int
 
 
@@ -91,7 +91,7 @@ def _getHeaders() -> dict[str, str]:
     if apiKey is None:
         apiKey = ""
     return {
-        "X-jinko-project-id": _projectId,
+        "X-jinko-project-id": _projectId,  # type: ignore
         "Authorization": "%s %s" % (AUTHORIZATION_PREFIX, apiKey),
         "User-Agent": USER_AGENT,
     }
@@ -128,9 +128,9 @@ def makeRequest(
     params=None,
     json=None,
     csv_data=None,
-    options: MakeRequestOptions = None,
+    options: MakeRequestOptions | None = None,
     data=None,
-):
+) -> _requests.Response:
     """Makes an HTTP request to the Jinko API.
 
     Args:
@@ -209,7 +209,8 @@ def makeRequest(
     else:
         data_param = None
 
-    headers["Content-Type"] = input_mime_type
+    if input_mime_type:
+        headers["Content-Type"] = input_mime_type
     if output_mime_type is not None:
         headers["Accept"] = output_mime_type
 
@@ -219,19 +220,19 @@ def makeRequest(
         _baseUrl + path,
         headers=headers,
         params=params,
-        **({data_param: data} if data_param else {}),
+        **({data_param: data} if data_param else {}),  # type: ignore
     )
     if response.status_code not in [200, 204]:
         if (
             "content-type" in response.headers
             and response.headers["content-type"] == "application/json"
         ):
-            print(response.json())
+            sys.stderr.write(response.json() + "\n")
         else:
-            print("%s: %s" % (response.status_code, response.text))
+            sys.stderr.write("%s: %s\n" % (response.status_code, response.text))
         response.raise_for_status()
     if response.status_code == 204:
-        return "Query successfully done, got a 204 response"
+        sys.stderr.write("Query successfully done, got a 204 response\n")
     return response
 
 
@@ -264,7 +265,7 @@ def nextPage(lastResponse: _requests.Response) -> _requests.Response | None:
 
 def fetchAllJson(
     path: str,
-) -> list[any]:
+) -> list[Any]:
     """Makes a GET HTTP request and retrieve all pages of a paginated response as json
 
     Args:
@@ -306,7 +307,7 @@ def checkAuthentication() -> bool:
     if response.status_code == 401:
         return False
     if response.status_code != 200:
-        print(response.json())
+        sys.stderr.write(response.json() + "\n")
         response.raise_for_status()
     return True
 
@@ -379,7 +380,7 @@ def initialize(
         _apiKey = _getpass.getpass("Please enter your API key")
     if _apiKey.strip() == "":
         message = "API key cannot be empty"
-        print(message)
+        sys.stderr.write(message + "\n")
         raise Exception(message)
 
     # Ask user for API key/projectId and check authentication
@@ -387,14 +388,14 @@ def initialize(
         _projectId = _getpass.getpass("Please enter your Project Id")
     if _projectId.strip() == "":
         message = "Project Id cannot be empty"
-        print(message)
+        sys.stderr.write(message + "\n")
         raise Exception(message)
 
     if not checkAuthentication():
         message = 'Authentication failed for Project "%s"' % (_projectId)
-        print(message)
+        sys.stderr.write(message + "\n")
         raise Exception(message)
-    print("Authentication successful")
+    sys.stderr.write("Authentication successful\n")
 
 
 def getProjectItem(shortId: str, revision: int | None = None) -> dict:
@@ -416,6 +417,11 @@ def getProjectItem(shortId: str, revision: int | None = None) -> dict:
 
         projectItem = jinko.getProjectItem('tr-EUsp-WjjI', 1)
     """
+    warnings.warn(
+        "The function get_project_item with the old signature is deprecated. "
+        "Please use the updated function with the ability to use coreItemId and snapshotId.",
+        category=DeprecationWarning,
+    )
     if revision is None:
         return makeRequest("/app/v1/project-item/%s" % (shortId)).json()
     else:
@@ -424,15 +430,15 @@ def getProjectItem(shortId: str, revision: int | None = None) -> dict:
         ).json()
 
 
-def getCoreItemId(shortId: str, revision: int | None = None) -> CoreItemId:
-    """Retrieves the CoreItemId corresponding to a ProjectItem
+def getCoreItemId(shortId: str, revision: int | None = None) -> CoreItemIdDict:
+    """Retrieves the CoreItemId dictionnary corresponding to a ProjectItem
 
     Args:
         shortId (str): short Id of the ProjectItem
         revision (int | None, optional): revision number. Defaults to None
 
     Returns:
-        CoreItemId: corresponding CoreItemId
+        CoreItemIdDict: corresponding CoreItemId dictionnary
 
     Raises:
         Exception: if HTTP status code is not 200
@@ -446,92 +452,11 @@ def getCoreItemId(shortId: str, revision: int | None = None) -> CoreItemId:
     item = getProjectItem(shortId, revision)
     if "coreId" not in item or item["coreId"] is None:
         message = 'ProjectItem "%s" has no CoreItemId' % (shortId)
-        print(message)
+        sys.stderr.write(message + "\n")
         raise Exception(message)
     return item["coreId"]
 
 
-def dataTableToSQLite(
-    data_table_file_path: str,
-):
-    """
-    Converts a CSV file to an SQLite database and encodes it in base64.
-
-    Args:
-        data_table_file_path (str): The path to the CSV file.
-
-    Returns:
-        str: The encoded SQLite database.
-
-    Raises:
-        FileNotFoundError: If the CSV file does not exist.
-
-    Examples:
-        >>> datTableToSQLite('path/to/data.csv')
-        'encoded_data_table'
-    """
-    # Step 1: Convert CSV to SQLite
-
-    # Read the CSV file
-    df = _pandas.read_csv(data_table_file_path)
-    column_real_names = df.columns.tolist()
-
-    # Rename columns to col_{i} with {i} > 0
-    column_names = ["col_%s" % (i + 1) for i, _ in enumerate(column_real_names)]
-    column_names_mapping = {}
-    for i, name in enumerate(column_real_names):
-        column_names_mapping[name] = column_names[i]
-    df.rename(columns=column_names_mapping, inplace=True)
-
-    # Connect to SQLite database (or create it)
-    data_table_sqlite_file_path = _os.path.splitext(data_table_file_path)[0] + ".sqlite"
-
-    # Remove existing file
-    try:
-        _os.remove(data_table_sqlite_file_path)
-    except:
-        pass
-
-    conn = _sqlite3.connect(data_table_sqlite_file_path)
-    cursor = conn.cursor()
-
-    # Write the DataFrame to the SQLite database
-    df.to_sql(
-        "data",
-        conn,
-        if_exists="replace",
-        index=False,
-        dtype="TEXT",
-    )
-
-    # Create the 'data_columns' table
-    cursor.execute(
-        """
-    CREATE TABLE IF NOT EXISTS data_columns (
-        name TEXT UNIQUE,
-        realname TEXT UNIQUE
-    )
-    """
-    )
-
-    # Insert column data into 'data_columns' table
-    for i, name in enumerate(column_names):
-        cursor.execute(
-            "INSERT OR IGNORE INTO data_columns (name, realname) VALUES (?, ?)",
-            (name, column_real_names[i]),
-        )
-
-    # Commit changes and close the connection
-    conn.commit()
-    conn.close()
-
-    # Step 2: Encode SQLite database in base64
-    with open(data_table_sqlite_file_path, "rb") as f:
-        encoded_data_table = _base64.b64encode(f.read()).decode("utf-8")
-
-    return encoded_data_table
-
-
 def getProjectItemInfoFromResponse(
     response: _requests.Response,
 ) -> ProjectItemInfoFromResponse | None:
@@ -628,7 +553,7 @@ def is_interactive():
     """Check if the environment supports interactive plot display (like Jupyter or IPython)."""
     try:
         # Check if in an IPython environment
-        __IPYTHON__
+        __IPYTHON__  # type: ignore
         return True
     except NameError:
         return False
@@ -651,3 +576,70 @@ def show_plot_conditionally(fig, file_name=None):
                 _os.unlink(file_name)
             raise
         print(f"Plot saved to {file_name} . Please open it in a web browser.")
+
+
+def list_project_item_versions(sid: str, only_labeled: bool = False):
+    """
+    Retrieve the list of versions of a given ProjectItem.
+
+    Args:
+        sid (str): The short ID of the ProjectItem.
+        only_labeled (bool, optional): If True, only return labeled versions. Defaults to False.
+
+    Returns:
+        List[ProjectItemVersion]: The list of versions of the ProjectItem.
+    """
+    only_labeled_str = "true" if only_labeled else "false"
+    response = makeRequest(
+        f"/app/v1/project-item/{sid}/versions",
+        params={"onlyLabeled": only_labeled_str},
+    )
+    return response.json()
+
+
+def get_project_item(
+    core_item_id=None,
+    snapshot_id=None,
+    sid: Optional[str] = None,
+    revision: Optional[float] = None,
+    label: Optional[str] = None,
+):
+    """
+    Retrieve a ProjectItem from its CoreItemId, snapshotId, or its SID and revision.
+
+    Args:
+        core_item_id (str, optional): The CoreItemId of the ProjectItem.
+        snapshot_id (str, optional): The snapshotId of the ProjectItem.
+        sid (str, optional): The SID of the ProjectItem.
+        revision (int, optional): The revision of the ProjectItem.
+        label (str, optional): The label of the ProjectItem.
+
+    Returns:
+        ProjectItem, optional: The retrieved ProjectItem, or None if not found.
+
+    Raises:
+        ValueError: If neither 'sid' nor 'core_item_id' is provided.
+        Exception: If the parameters are ambiguous, i.e. they do not point to the same project item.
+    """
+
+    if core_item_id:
+        return makeRequest(
+            path=f"/app/v1/core-item/{core_item_id}", params={"snapshotId": snapshot_id}
+        ).json()
+    elif sid:
+        if revision:
+            return makeRequest(
+                path=f"/app/v1/project-item/{sid}", params={"revision": revision}
+            ).json()
+        elif label:
+            labeled_versions = makeRequest(
+                path=f"/app/v1/project-item/{sid}/versions?onlyLabeled=true"
+            ).json()
+            try:
+                return next(x for x in labeled_versions if x["label"] == label)
+            except StopIteration:
+                raise ValueError(f"Unknown version label {label}")
+        else:
+            return makeRequest(path=f"/app/v1/project-item/{sid}").json()
+    else:
+        raise ValueError("You must provide either 'sid' or 'core_item_id'")
